@@ -1,7 +1,7 @@
 #  File src/library/stats/R/density.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 density <- function(x, ...) UseMethod("density")
 
@@ -26,7 +26,7 @@ density.default <-
 	     give.Rkern = FALSE,
 	     n = 512, from, to, cut = 3, na.rm = FALSE, ...)
 {
-    if(!missing(...)) warning("non-matched further arguments are disregarded")
+    chkDots(...)
     if(!missing(window) && missing(kernel))
 	kernel <- window
     kernel <- match.arg(kernel)
@@ -52,7 +52,7 @@ density.default <-
         else stop("'x' contains missing values")
     }
     N <- nx <- as.integer(length(x))
-    if(is.na(N)) stop("invalid value of length(x)")
+    if(is.na(N)) stop(gettextf("invalid value of %s", "length(x)"), domain = NA)
     x.finite <- is.finite(x)
     if(any(!x.finite)) {
         x <- x[x.finite]
