@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1998-2019   The R Core Team.
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2017   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,6 +55,8 @@
  *	"verbose"
  *	"keep.source"
  *	"keep.source.pkgs"
+ *	"keep.parse.data"
+ *	"keep.parse.data.pkgs"
  *	"browserNLdisabled"
 
  *	"de.cellwidth"		../unix/X11/ & ../gnuwin32/dataentry.c
@@ -64,6 +66,8 @@
 
  *	"timeout"		./connections.c
 
+ *      "deparse.max.lines"     ./deparse.c (& PrintCall() in ./eval.c, ./main.c
+
  *	"check.bounds"
  *	"error"
  *	"error.messages"
@@ -84,6 +88,9 @@
  * "object.size"
  * "reference", "show"
  * "scrap"
+
+ * R_NilValue is not a valid value for any option, but is used to signal a
+ * missing option by FindTaggedItem/GetOption and higher-level functions.
  */
 
 
@@ -97,8 +104,11 @@ static SEXP Options(void)
 static SEXP FindTaggedItem(SEXP lst, SEXP tag)
 {
     for ( ; lst != R_NilValue ; lst = CDR(lst)) {
-	if (TAG(lst) == tag)
+	if (TAG(lst) == tag) {
+	    if (CAR(lst) == R_NilValue) 
+		error("option %s has NULL value", CHAR(PRINTNAME(tag)));
 	    return lst;
+	}
     }
     return R_NilValue;
 }
@@ -247,10 +257,11 @@ void attribute_hidden InitOptions(void)
     SEXP val, v;
     char *p;
 
+    /* options set here should be included into mandatory[] in do_options */
 #ifdef HAVE_RL_COMPLETION_MATCHES
-    PROTECT(v = val = allocList(21));
+    PROTECT(v = val = allocList(23));
 #else
-    PROTECT(v = val = allocList(20));
+    PROTECT(v = val = allocList(22));
 #endif
 
     SET_TAG(v, install("prompt"));
@@ -292,7 +303,7 @@ void attribute_hidden InitOptions(void)
     p = getenv("R_KEEP_PKG_SOURCE");
     R_KeepSource = (p && (strcmp(p, "yes") == 0)) ? 1 : 0;
 
-    SET_TAG(v, install("keep.source")); /* overridden in common.R */
+    SET_TAG(v, install("keep.source")); /* overridden in Common.R */
     SETCAR(v, ScalarLogical(R_KeepSource));
     v = CDR(v);
 
@@ -300,6 +311,15 @@ void attribute_hidden InitOptions(void)
     SETCAR(v, ScalarLogical(R_KeepSource));
     v = CDR(v);
 
+    SET_TAG(v, install("keep.parse.data"));
+    SETCAR(v, ScalarLogical(TRUE));
+    v = CDR(v);
+
+    p = getenv("R_KEEP_PKG_PARSE_DATA");
+    SET_TAG(v, install("keep.parse.data.pkgs"));
+    SETCAR(v, ScalarLogical((p && (strcmp(p, "yes") == 0)) ? TRUE : FALSE));
+    v = CDR(v);
+
     SET_TAG(v, install("warning.length"));
     SETCAR(v, ScalarInteger(1000));
     v = CDR(v);
@@ -334,9 +354,9 @@ void attribute_hidden InitOptions(void)
     v = CDR(v);
 
     SET_TAG(v, install("PCRE_study"));
-    if (R_PCRE_study == -1) 
+    if (R_PCRE_study == -1)
 	SETCAR(v, ScalarLogical(TRUE));
-    else if (R_PCRE_study == -2) 
+    else if (R_PCRE_study == -2)
 	SETCAR(v, ScalarLogical(FALSE));
     else
 	SETCAR(v, ScalarInteger(R_PCRE_study));
@@ -350,6 +370,7 @@ void attribute_hidden InitOptions(void)
     R_PCRE_limit_recursion = NA_LOGICAL;
     SETCAR(v, ScalarLogical(R_PCRE_limit_recursion));
     v = CDR(v);
+    /* options set here should be included into mandatory[] in do_options */
 
 #ifdef HAVE_RL_COMPLETION_MATCHES
     /* value from Rf_initialize_R */
@@ -471,7 +492,28 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 	if (*CHAR(namei)) { /* name = value  ---> assignment */
 	    SEXP tag = installTrChar(namei);
-	    if (streql(CHAR(namei), "width")) {
+	    SET_STRING_ELT(names, i, namei);
+
+	    if (argi == R_NilValue) {
+		/* Handle option removal separately to simplify value checking
+		   for known options below; mandatory means not allowed to be
+		   removed once set, but not all have to be set at startup. */
+		const char *mandatory[] = {"prompt", "continue", "expressions",
+		  "width", "deparse.cutoff", "digits", "echo", "verbose",
+		  "check.bounds", "keep.source", "keep.source.pkgs",
+		  "keep.parse.data", "keep.parse.data.pkgs", "warning.length",
+		  "nwarnings", "OutDec", "browserNLdisabled", "CBoundsCheck",
+		  "matprod", "PCRE_study", "PCRE_use_JIT",
+		  "PCRE_limit_recursion", "rl_word_breaks",
+		  /* ^^^ from InitOptions ^^^ */
+		  "warn", "max.print", "show.error.messages",
+		  /* ^^^ from Common.R ^^^ */
+		  NULL};
+		for(int j = 0; mandatory[j] != NULL; j++)
+		    if (streql(CHAR(namei), mandatory[j]))
+			error(_("option '%s' cannot be deleted"), CHAR(namei));
+		SET_VECTOR_ELT(value, i, SetOption(tag, R_NilValue));
+	    } else if (streql(CHAR(namei), "width")) {
 		int k = asInteger(argi);
 		if (k < R_MIN_WIDTH_OPT || k > R_MAX_WIDTH_OPT)
 		    error(_("invalid 'width' parameter, allowed %d...%d"),
@@ -703,34 +745,33 @@ SEXP attribute_hidden do_options(SEXP call, SEXP op, SEXP args, SEXP rho)
 		if (TYPEOF(argi) == LGLSXP) {
 		    int k = asLogical(argi) > 0;
 		    R_PCRE_study = k ? -1 : -2;
-		    SET_VECTOR_ELT(value, i, 
+		    SET_VECTOR_ELT(value, i,
 				   SetOption(tag, ScalarLogical(k)));
 		} else {
 		    R_PCRE_study = asInteger(argi);
 		    if (R_PCRE_study < 0) {
 			R_PCRE_study = -2;
-			SET_VECTOR_ELT(value, i, 
+			SET_VECTOR_ELT(value, i,
 				       SetOption(tag, ScalarLogical(-2)));
 		    } else
-			SET_VECTOR_ELT(value, i, 
+			SET_VECTOR_ELT(value, i,
 				       SetOption(tag, ScalarInteger(R_PCRE_study)));
 		}
 	    }
 	    else if (streql(CHAR(namei), "PCRE_use_JIT")) {
 		int use_JIT = asLogical(argi);
 		R_PCRE_use_JIT = (use_JIT > 0); // NA_LOGICAL is < 0
-		SET_VECTOR_ELT(value, i, 
+		SET_VECTOR_ELT(value, i,
 			       SetOption(tag, ScalarLogical(R_PCRE_use_JIT)));
 	    }
 	    else if (streql(CHAR(namei), "PCRE_limit_recursion")) {
 		R_PCRE_limit_recursion = asLogical(argi);
-		SET_VECTOR_ELT(value, i, 
+		SET_VECTOR_ELT(value, i,
 			       SetOption(tag, ScalarLogical(R_PCRE_limit_recursion)));
 	    }
 	    else {
 		SET_VECTOR_ELT(value, i, SetOption(tag, duplicate(argi)));
 	    }
-	    SET_STRING_ELT(names, i, namei);
 	}
 	else { /* querying arg */
 	    const char *tag;
